#
# Copyright (c) 2015-2016 LAAS/CNRS
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software for any purpose
# with or without   fee is hereby granted, provided   that the above  copyright
# notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS  SOFTWARE INCLUDING ALL  IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS. IN NO EVENT SHALL THE AUTHOR  BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR  ANY DAMAGES WHATSOEVER RESULTING  FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION,   ARISING OUT OF OR IN    CONNECTION WITH THE USE   OR
# PERFORMANCE OF THIS SOFTWARE.
#
#                                             Anthony Mallet on Mon May 18 2015
#
import server
from util import *

_id = 0
_sessions = {}

keys = {}


# --- start --------------------------------------------------------------
#
def start(channel, params):
  global _id
  global _sessions

  s = get(params)
  if s is None:
    while True:
      _id += 1
      s = "s%d" % _id
      if not s in _sessions: break

  if s in _sessions:
    raise softerror("session %s exists" % s)

  # infox "$channel: new session $s"
  _sessions[s] = {'events': []}
  return s

keys['start'] = start


# --- kill -----------------------------------------------------------------
#
def kill():
  global _sessions

  for s in _sessions:
    infox('closing session %s' % s)
    server.push(_sessions[s]['listen'], 'quit')

  _sessions = {}


# --- get ------------------------------------------------------------------
#
def get(params):
  if 's' in params: return params['s'][0] if params['s'][0] else None
  return None


# --- get_channel ----------------------------------------------------------
#
def get_channel(params):
  global _sessions

  s = get(params)
  if s in _sessions:
    if 'listen' in _sessions[s]: return _sessions[s]['listen']

  return None


# --- broadcast ------------------------------------------------------------
#
def broadcast(event, data = ''):
  global _sessions

  for s in _sessions:
    if 'listen' in _sessions[s]:
      server.push(_sessions[s]['listen'], event, data)


# --- listen ---------------------------------------------------------------
#
def listen(channel, params):
  global _sessions

  s = get(params)
  if s is None: return
  if s not in _sessions:
    raise softerror("session %s does not exist" % s)

  if 'listen' in _sessions[s]:
    infox("%s: terminating %s listener on session %s" %
          (channel, _sessions[s]['listen'], s))
    server.push(_sessions[s]['listen'], 'quit')

  infox("%s: listening session %s" % (channel, s))
  _sessions[s]['listen'] = channel

keys['listen'] = listen
